@@ -49,7 +49,9 @@ import net.i2p.util.SystemVersion;
 import org.eclipse.jetty.security.HashLoginService;
 import org.eclipse.jetty.security.ConstraintMapping;
 import org.eclipse.jetty.security.ConstraintSecurityHandler;
+import org.eclipse.jetty.security.authentication.BasicAuthenticator;
 import org.eclipse.jetty.security.authentication.DigestAuthenticator;
+import org.eclipse.jetty.security.authentication.LoginAuthenticator;
 import org.eclipse.jetty.server.AbstractConnector;
 import org.eclipse.jetty.server.ConnectionFactory;
 import org.eclipse.jetty.server.Connector;
@@ -116,6 +118,9 @@ public class RouterConsoleRunner implements RouterApp {
 
     // Jetty Auth
     private static final DigestAuthenticator authenticator = new DigestAuthenticator();
+    // only for prometheus plugin
+    private static final BasicAuthenticator basicAuthenticator = new BasicAuthenticator();
+    public static final String PROMETHEUS_REALM = "prometheus";
     static {
         // default changed from 0 (forever) in Jetty 6 to 60*1000 ms in Jetty 7
         authenticator.setMaxNonceAge(7*24*60*60*1000L);
@@ -366,13 +371,10 @@ public class RouterConsoleRunner implements RouterApp {
     /** @since 0.9.17 */
     private void checkJavaVersion() {
         boolean noJava8 = !SystemVersion.isJava8();
-        //boolean noPack200 = (PluginStarter.pluginsEnabled(_context) || !NewsHelper.isUpdateDisabled(_context)) &&
-        //                    !FileUtil.isPack200Supported();
-        boolean noPack200 = false;
         boolean openARM = SystemVersion.isARM() && SystemVersion.isOpenJDK() && !SystemVersion.isJava9();
         boolean isZero = SystemVersion.isZeroVM();
-        boolean isJava11 = false; // SystemVersion.isJava11();
-        if (noJava8 || noPack200 || openARM || isZero || isJava11) {
+        boolean isJava17 = SystemVersion.isJava(17);
+        if (noJava8 || openARM || isZero || !isJava17) {
             String s = "Java version: " + System.getProperty("java.version") +
                        " OS: " + System.getProperty("os.name") + ' ' +
                        System.getProperty("os.arch") + ' ' +
@@ -380,16 +382,13 @@ public class RouterConsoleRunner implements RouterApp {
             net.i2p.util.Log log = _context.logManager().getLog(RouterConsoleRunner.class);
             log.logAlways(net.i2p.util.Log.WARN, s);
             System.out.println("Warning: " + s);
-            if (noJava8) {
-                s = "Java 8 or higher is required, please upgrade Java";
+            if (!isJava17) {
+                s = "Java 17 or higher will be required in a future release, please upgrade Java now to continue receiving I2P updates";
                 log.logAlways(net.i2p.util.Log.WARN, s);
                 System.out.println("Warning: " + s);
             }
-            if (noPack200) {
-                if (SystemVersion.isJava(14))
-                    s = "Pack200 is required for some plugins, please consider downgrading Java to 13 or lower";
-                else
-                    s = "Pack200 is required for some plugins, please consider upgrading Java";
+            if (noJava8) {
+                s = "Java 8 or higher is required, please upgrade Java";
                 log.logAlways(net.i2p.util.Log.WARN, s);
                 System.out.println("Warning: " + s);
             }
@@ -985,15 +984,25 @@ public class RouterConsoleRunner implements RouterApp {
                 enable = false;
                 ctx.router().saveConfig(PROP_PW_ENABLE, "false");
             } else {
-                HashLoginService realm = new CustomHashLoginService(JETTY_REALM, context.getContextPath(),
+                // Prometheus server only supports basic auth
+                // https://github.com/prometheus/common/issues/352
+                // Jetty does not support multiple auth at once
+                // but it's coming for Jetty 12
+                // https://github.com/jetty/jetty.project/issues/5442
+                boolean isBasic = context.getContextPath().equals("/prometheus");
+                // need separate realms so the browser doesn't get them mixed up
+                String rlm = isBasic ? PROMETHEUS_REALM : JETTY_REALM;
+                HashLoginService realm = new CustomHashLoginService(rlm, context.getContextPath(),
                                                                     ctx.logManager().getLog(RouterConsoleRunner.class));
                 sec.setLoginService(realm);
-                sec.setAuthenticator(authenticator);
+                LoginAuthenticator auth = isBasic ? basicAuthenticator : authenticator;
+                sec.setAuthenticator(auth);
                 String[] role = new String[] {JETTY_ROLE};
                 for (Map.Entry<String, String> e : userpw.entrySet()) {
                     String user = e.getKey();
                     String pw = e.getValue();
-                    Credential cred = Credential.getCredential(MD5_CREDENTIAL_TYPE + pw);
+                    // for basic, the password will be the md5 hash itself
+                    Credential cred = Credential.getCredential(isBasic ? pw : MD5_CREDENTIAL_TYPE + pw);
                     realm.putUser(user, cred, role);
                     Constraint constraint = new Constraint(user, JETTY_ROLE);
                     constraint.setAuthenticate(true);
@@ -1104,7 +1113,7 @@ public class RouterConsoleRunner implements RouterApp {
             UserIdentity rv = super.login(username, credentials, request);
             if (rv == null)
                 //_log.logAlways(net.i2p.util.Log.WARN, "Console authentication failed, webapp: " + _webapp + ", user: " + username);
-                _log.logAlways(net.i2p.util.Log.WARN, "Console authentication failed, user: " + username);
+                _log.logAlways(net.i2p.util.Log.WARN, "Console authentication failed, user: " + username + " IP: " + request.getRemoteAddr());
             return rv;
         }
     }
