@@ -48,6 +48,7 @@ import net.i2p.data.SimpleDataStructure;
 import net.i2p.data.i2cp.I2CPMessage;
 import net.i2p.data.i2cp.RequestLeaseSetMessage;
 import net.i2p.util.Log;
+import net.i2p.util.OrderedProperties;
 
 /**
  * Handle I2CP RequestLeaseSetMessage from the router by granting all leases,
@@ -144,6 +145,25 @@ class RequestLeaseSetMessageHandler extends HandlerImpl {
             }
             if (Boolean.parseBoolean(session.getOptions().getProperty("i2cp.dontPublishLeaseSet")))
                 ls2.setUnpublished();
+
+            // Service records, proposal 167
+            String k = "i2cp.leaseSetOption.0";
+            Properties props = null;
+            for (int i = 0; i < 10; i++) {
+                String v = session.getOptions().getProperty(k);
+                if (v == null)
+                    break;
+                String[] vs = DataHelper.split(v, "=", 2);
+                if (vs.length < 2)
+                    continue;
+                if (props == null)
+                    props = new OrderedProperties();
+                props.setProperty(vs[0], vs[1]);
+                k = "i2cp.leaseSetOption." + (i + 1);
+            }
+            if (props != null)
+                ls2.setOptions(props);
+
             // ensure 1-second resolution timestamp is higher than last one
             long now = Math.max(_context.clock().now(), session.getLastLS2SignTime() + 1000);
             ls2.setPublished(now);
