@@ -41,17 +41,26 @@ class SymmetricState implements Destroyable, Cloneable {
 	private static final byte[] INIT_CK_IK;
 	private static final byte[] INIT_CK_N;
 	private static final byte[] INIT_CK_XK_SSU2;
+	private static final byte[] INIT_CK_IKHFS_512;
+	private static final byte[] INIT_CK_IKHFS_768;
+	private static final byte[] INIT_CK_IKHFS_1024;
 	// precalculated hash of the hash of the Noise name = mixHash(nullPrologue)
 	private static final byte[] INIT_HASH_XK = new byte[32];
 	private static final byte[] INIT_HASH_IK = new byte[32];
 	private static final byte[] INIT_HASH_N = new byte[32];
 	private static final byte[] INIT_HASH_XK_SSU2 = new byte[32];
+	private static final byte[] INIT_HASH_IKHFS_512 = new byte[32];
+	private static final byte[] INIT_HASH_IKHFS_768 = new byte[32];
+	private static final byte[] INIT_HASH_IKHFS_1024 = new byte[32];
 
 	static {
 		INIT_CK_XK = initHash(HandshakeState.protocolName);
 		INIT_CK_IK = initHash(HandshakeState.protocolName2);
 		INIT_CK_N = initHash(HandshakeState.protocolName3);
 		INIT_CK_XK_SSU2 = initHash(HandshakeState.protocolName4);
+	        INIT_CK_IKHFS_512 = initHash(HandshakeState.protocolName5);
+	        INIT_CK_IKHFS_768 = initHash(HandshakeState.protocolName6);
+	        INIT_CK_IKHFS_1024 = initHash(HandshakeState.protocolName7);
 		try {
 			MessageDigest md = Noise.createHash("SHA256");
 			md.update(INIT_CK_XK, 0, 32);
@@ -62,6 +71,12 @@ class SymmetricState implements Destroyable, Cloneable {
 			md.digest(INIT_HASH_N, 0, 32);
 			md.update(INIT_CK_XK_SSU2, 0, 32);
 			md.digest(INIT_HASH_XK_SSU2, 0, 32);
+			md.update(INIT_CK_IKHFS_512, 0, 32);
+			md.digest(INIT_HASH_IKHFS_512, 0, 32);
+			md.update(INIT_CK_IKHFS_768, 0, 32);
+			md.digest(INIT_HASH_IKHFS_768, 0, 32);
+			md.update(INIT_CK_IKHFS_1024, 0, 32);
+			md.digest(INIT_HASH_IKHFS_1024, 0, 32);
 			Noise.releaseHash(md);
 		} catch (Exception e) {
 			throw new IllegalStateException(e);
@@ -136,6 +151,15 @@ class SymmetricState implements Destroyable, Cloneable {
 		} else if (patternId.equals(HandshakeState.PATTERN_ID_XK_SSU2)) {
 			initCK = INIT_CK_XK_SSU2;
 			initHash = INIT_HASH_XK_SSU2;
+		} else if (patternId.equals(HandshakeState.PATTERN_ID_IKHFS_512)) {
+			initCK = INIT_CK_IKHFS_512;
+			initHash = INIT_HASH_IKHFS_512;
+		} else if (patternId.equals(HandshakeState.PATTERN_ID_IKHFS_768)) {
+			initCK = INIT_CK_IKHFS_768;
+			initHash = INIT_HASH_IKHFS_768;
+		} else if (patternId.equals(HandshakeState.PATTERN_ID_IKHFS_1024)) {
+			initCK = INIT_CK_IKHFS_1024;
+			initHash = INIT_HASH_IKHFS_1024;
 		} else {
 			throw new IllegalArgumentException("Handshake pattern is not recognized");
 		}
@@ -319,6 +343,7 @@ class SymmetricState implements Destroyable, Cloneable {
 	 */
 	public int decryptAndHash(byte[] ciphertext, int ciphertextOffset, byte[] plaintext, int plaintextOffset, int length) throws ShortBufferException, BadPaddingException
 	{
+		// NOTE: This updates the hash, even on failure
 		System.arraycopy(h, 0, prev_h, 0, h.length);
 		mixHash(ciphertext, ciphertextOffset, length);
 		return cipher.decryptWithAd(prev_h, ciphertext, ciphertextOffset, plaintext, plaintextOffset, length);
