@@ -43,6 +43,7 @@ import net.i2p.data.SigningPrivateKey;
 import net.i2p.data.SigningPublicKey;
 import net.i2p.data.i2np.GarlicMessage;
 import net.i2p.data.router.RouterAddress;
+import net.i2p.data.router.RouterIdentity;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.router.CommSystemFacade.Status;
 import net.i2p.router.crypto.FamilyKeyCrypto;
@@ -94,6 +95,8 @@ public class Router implements RouterClock.ClockShiftListener {
     private String _configFilename;
     private RouterInfo _routerInfo;
     private final ReentrantReadWriteLock _routerInfoLock = new ReentrantReadWriteLock(false);
+    private RouterIdentity _routerIdent;
+    private Hash _routerHash;
     /** not for external use */
     public final Object routerInfoFileLock = new Object();
     private final Object _configFileLock = new Object();
@@ -583,6 +586,26 @@ public class Router implements RouterClock.ClockShiftListener {
         }
     }
 
+    /**
+     *  Our current router identity.
+     *  Warning, may be null if called very early.
+     *  Lockless.
+     *  @since 0.9.67
+     */
+    public RouterIdentity getRouterIdentity() {
+        return _routerIdent;
+    }
+
+    /**
+     *  Our current router hash.
+     *  Warning, may be null if called very early.
+     *  Lockless.
+     *  @since 0.9.67
+     */
+    public Hash getRouterHash() {
+        return _routerHash;
+    }
+
     /**
      *  Caller must ensure info is valid - no validation done here.
      *  Not for external use.
@@ -592,6 +615,12 @@ public class Router implements RouterClock.ClockShiftListener {
      */
     public void setRouterInfo(RouterInfo info) { 
         _routerInfoLock.writeLock().lock();
+        if (!info.getIdentity().equals(_routerIdent)) {
+            if (_routerIdent != null)  // shouldn't happen
+                _log.log(Log.CRIT, "Changing router ident while running");
+            _routerIdent = info.getIdentity();
+            _routerHash = _routerIdent.calculateHash();
+        }
         try {
             _routerInfo = info; 
         } finally {
@@ -758,7 +787,7 @@ public class Router implements RouterClock.ClockShiftListener {
         synchronized(_configFileLock) {
             String f = getConfigFilename();
             Properties config = getConfig(_context, f);
-            // to avoid compiler errror
+            // to avoid compiler error
             Map foo = _config;
             foo.putAll(config);
         }
@@ -927,23 +956,23 @@ public class Router implements RouterClock.ClockShiftListener {
                 changed = true;
             }
         }
-        if (changed && _context.netDb().isInitialized()) {
+        if (changed) {
             if (_log.shouldWarn())
-                _log.warn("NetDB ready, publishing RI");
+                _log.warn("NetDB ready, initialized? " + _context.netDb().isInitialized());
             // any previous calls to netdb().publish() did not
             // actually publish, because netdb init was not complete
             Republish r = new Republish(_context);
             // this is called from PersistentDataStore.ReadJob,
             // so we probably don't need to throw it to the timer queue,
             // but just to be safe
-            _context.simpleTimer2().addEvent(r, 0);
+            long delay = _context.netDb().isInitialized() ? 0 : 60*1000;
+            _context.simpleTimer2().addEvent(r, delay);
 
             // periodically update our RI and republish it to the flooodfills
             PublishLocalRouterInfoJob plrij = new PublishLocalRouterInfoJob(_context);
             plrij.getTiming().setStartAfter(_context.clock().now() + plrij.getDelay());
             _context.jobQueue().addJob(plrij);
-        }
-        if (changed) {
+
             _context.commSystem().initGeoIP();
 
             if (!SystemVersion.isSlow() &&
