@@ -863,7 +863,13 @@ class ClientManager {
         public void timeReached() {
             if (!_isStarted)
                 return;
-            for (ClientConnectionRunner runner : _runners.values()) {
+            if (_runners.isEmpty()) {
+                schedule(LOOP_TIME);
+                return;
+            }
+            // dedup subsessions
+            Set<ClientConnectionRunner> runners = new HashSet<ClientConnectionRunner>(_runners.values());
+            for (ClientConnectionRunner runner : runners) {
                 if (runner instanceof QueuedClientConnectionRunner)
                     continue;
                 if (runner.isDead())
