@@ -16,6 +16,7 @@ import com.southernstorm.noise.protocol.HandshakeState;
 
 import net.i2p.crypto.EncType;
 import net.i2p.crypto.HKDF;
+import net.i2p.crypto.KeyFactory;
 import net.i2p.crypto.SessionKeyManager;
 import net.i2p.data.Base64;
 import net.i2p.data.Certificate;
@@ -33,6 +34,7 @@ import net.i2p.data.SessionTag;
 import net.i2p.data.i2np.DatabaseStoreMessage;
 import net.i2p.data.i2np.GarlicClove;
 import net.i2p.data.i2np.I2NPMessage;
+import net.i2p.router.crypto.pqc.MLKEM;
 import static net.i2p.router.crypto.ratchet.RatchetPayload.*;
 import net.i2p.router.LeaseSetKeys;
 import net.i2p.router.Router;
@@ -53,6 +55,7 @@ public final class ECIESAEADEngine {
     private final RouterContext _context;
     private final Log _log;
     private final MuxedEngine _muxedEngine;
+    private final MuxedPQEngine _muxedPQEngine;
     private final HKDF _hkdf;
     private final Elg2KeyFactory _edhThread;
     private boolean _isRunning;
@@ -86,6 +89,24 @@ public final class ECIESAEADEngine {
     private static final String INFO_0 = "SessionReplyTags";
     private static final String INFO_6 = "AttachPayloadKDF";
 
+    // These are the min sizes for the MLKEM New Session Message.
+    // It contains an extra MLKEM key and MAC.
+    // 112
+    private static final int NS_MLKEM_OVERHEAD = NS_OVERHEAD + MACLEN;
+    // 800 + 112 + 7 = 919
+    private static final int MIN_NS_MLKEM512_SIZE = EncType.MLKEM512_X25519_INT.getPubkeyLen() + NS_MLKEM_OVERHEAD + DATETIME_SIZE;
+    // 1184 + 112 + 7 = 1303
+    private static final int MIN_NS_MLKEM768_SIZE = EncType.MLKEM768_X25519_INT.getPubkeyLen() + NS_MLKEM_OVERHEAD + DATETIME_SIZE;
+    // 1568 + 112 + 7 = 1687
+    private static final int MIN_NS_MLKEM1024_SIZE = EncType.MLKEM1024_X25519_INT.getPubkeyLen() + NS_MLKEM_OVERHEAD + DATETIME_SIZE;
+    // 856
+    private static final int MIN_NSR_MLKEM512_SIZE = EncType.MLKEM512_X25519_CT.getPubkeyLen() + MIN_NSR_SIZE + MACLEN;
+    // 1176
+    private static final int MIN_NSR_MLKEM768_SIZE = EncType.MLKEM768_X25519_CT.getPubkeyLen() + MIN_NSR_SIZE + MACLEN;
+    // 1656
+    private static final int MIN_NSR_MLKEM1024_SIZE = EncType.MLKEM1024_X25519_CT.getPubkeyLen() + MIN_NSR_SIZE + MACLEN;
+
+
     /**
      *  Caller MUST call startup() to get threaded generation.
      *  Will still work without, will just generate inline.
@@ -96,6 +117,7 @@ public final class ECIESAEADEngine {
         _context = ctx;
         _log = _context.logManager().getLog(ECIESAEADEngine.class);
         _muxedEngine = new MuxedEngine(ctx);
+        _muxedPQEngine = new MuxedPQEngine(ctx);
         _hkdf = new HKDF(ctx);
         _edhThread = new Elg2KeyFactory(ctx);
         
@@ -147,6 +169,18 @@ public final class ECIESAEADEngine {
         return _muxedEngine.decrypt(data, elgKey, ecKey, keyManager);
     }
 
+    /**
+     * Try to decrypt the message with one or both of the given private keys
+     *
+     * @param ecKey must be EC, non-null
+     * @param pqKey must be PQ, non-null
+     * @return decrypted data or null on failure
+     * @since 0.9.67
+     */
+    public CloveSet decrypt(byte data[], PrivateKey ecKey, PrivateKey pqKey, MuxedPQSKM keyManager) throws DataFormatException {
+        return _muxedPQEngine.decrypt(data, ecKey, pqKey, keyManager);
+    }
+
     /**
      * Decrypt the message using the given private key
      * and using tags from the specified key manager.
@@ -175,8 +209,7 @@ public final class ECIESAEADEngine {
 
     private CloveSet x_decrypt(byte data[], PrivateKey targetPrivateKey,
                                RatchetSKM keyManager) throws DataFormatException {
-        if (targetPrivateKey.getType() != EncType.ECIES_X25519)
-            throw new IllegalArgumentException();
+        checkType(targetPrivateKey.getType());
         if (data == null) {
             if (_log.shouldLog(Log.ERROR)) _log.error("Null data being decrypted?");
             return null;
@@ -267,14 +300,22 @@ public final class ECIESAEADEngine {
             if (shouldDebug)
                 _log.debug("Decrypting ES with tag: " + st.toBase64() + " key: " + key + ": " + data.length + " bytes");
             decrypted = decryptExistingSession(tag, data, key, targetPrivateKey, keyManager);
-        } else if (data.length >= MIN_NSR_SIZE) {
-            if (shouldDebug)
-                _log.debug("Decrypting NSR with tag: " + st.toBase64() + " key: " + key + ": " + data.length + " bytes");
-            decrypted = decryptNewSessionReply(tag, data, state, keyManager);
         } else {
-            decrypted = null;
-            if (_log.shouldWarn())
-                _log.warn("ECIES decrypt fail, tag found but no state and too small for NSR: " + data.length + " bytes");
+            // it's important not to attempt decryption for too-short packets,
+            // because Noise will destroy() the handshake state on failure,
+            // and we don't clone() on the first one, so it's fatal.
+            EncType type = targetPrivateKey.getType();
+            int min = getMinNSRSize(type);
+            if (data.length >= min) {
+                if (shouldDebug)
+                    _log.debug("Decrypting NSR with tag: " + st.toBase64() + " key: " + key + ": " + data.length + " bytes");
+                decrypted = decryptNewSessionReply(tag, data, state, keyManager);
+            } else {
+                decrypted = null;
+                if (_log.shouldWarn())
+                    _log.warn("NSR decrypt fail, tag: " + st.toBase64() + " but packet too small: " + data.length + " bytes, min is " +
+                              min + " on state " + state);
+            }
         }
         if (decrypted != null) {
             _context.statManager().updateFrequency("crypto.eciesAEAD.decryptExistingSession");
@@ -316,8 +357,10 @@ public final class ECIESAEADEngine {
     private CloveSet x_decryptSlow(byte data[], PrivateKey targetPrivateKey,
                                    RatchetSKM keyManager) throws DataFormatException {
         CloveSet decrypted;
+        EncType type = targetPrivateKey.getType();
+        int minns = getMinNSSize(type);
         boolean isRouter = keyManager.getDestination() == null;
-        if (data.length >= MIN_NS_SIZE || (isRouter && data.length >= MIN_NS_N_SIZE)) {
+        if (data.length >= minns || (isRouter && data.length >= MIN_NS_N_SIZE)) {
             if (isRouter)
                 decrypted = decryptNewSession_N(data, targetPrivateKey, keyManager);
             else
@@ -333,11 +376,116 @@ public final class ECIESAEADEngine {
         } else {
             decrypted = null;
             if (_log.shouldDebug())
-                _log.debug("ECIES decrypt fail, too small for NS: " + data.length + " bytes");
+                _log.debug("ECIES decrypt fail, too small for NS: " + data.length + " bytes, min is " +
+                           minns + " for type " + type);
         }
         return decrypted;
     }
 
+    /**
+     * @throws IllegalArgumentException if unsupported
+     * @since 0.9.67
+     */
+    private static void checkType(EncType type) {
+        switch(type) {
+          case ECIES_X25519:
+          case MLKEM512_X25519:
+          case MLKEM768_X25519:
+          case MLKEM1024_X25519:
+              return;
+          default:
+              throw new IllegalArgumentException("Unsupported key type " + type);
+        }
+    }
+
+    /**
+     * @since 0.9.67
+     */
+    private static String getNoisePattern(EncType type) {
+        switch(type) {
+          case ECIES_X25519:
+              return HandshakeState.PATTERN_ID_IK;
+          case MLKEM512_X25519:
+              return HandshakeState.PATTERN_ID_IKHFS_512;
+          case MLKEM768_X25519:
+              return HandshakeState.PATTERN_ID_IKHFS_768;
+          case MLKEM1024_X25519:
+              return HandshakeState.PATTERN_ID_IKHFS_1024;
+          default:
+              throw new IllegalArgumentException("No pattern for " + type);
+        }
+    }
+
+    /**
+     * @since 0.9.67
+     */
+    private static KeyFactory getHybridKeyFactory(EncType type) {
+        switch(type) {
+          case MLKEM512_X25519:
+              return MLKEM.MLKEM512KeyFactory;
+          case MLKEM768_X25519:
+              return MLKEM.MLKEM768KeyFactory;
+          case MLKEM1024_X25519:
+              return MLKEM.MLKEM1024KeyFactory;
+          default:
+              return null;
+        }
+    }
+
+    /**
+     * @since 0.9.67
+     */
+    private static int getMinNSSize(EncType type) {
+        switch(type) {
+          case ECIES_X25519:
+              return MIN_NS_SIZE;
+          case MLKEM512_X25519:
+              return MIN_NS_MLKEM512_SIZE;
+          case MLKEM768_X25519:
+              return MIN_NS_MLKEM768_SIZE;
+          case MLKEM1024_X25519:
+              return MIN_NS_MLKEM1024_SIZE;
+          default:
+              throw new IllegalArgumentException("No pattern for " + type);
+        }
+    }
+
+    /**
+     * @since 0.9.67
+     */
+    private static int getMinNSRSize(EncType type) {
+        switch(type) {
+          case ECIES_X25519:
+              return MIN_NSR_SIZE;
+          case MLKEM512_X25519:
+              return MIN_NSR_MLKEM512_SIZE;
+          case MLKEM768_X25519:
+              return MIN_NSR_MLKEM768_SIZE;
+          case MLKEM1024_X25519:
+              return MIN_NSR_MLKEM1024_SIZE;
+          default:
+              throw new IllegalArgumentException("No pattern for " + type);
+        }
+    }
+
+    /**
+     * @since 0.9.67
+     */
+    private static Set<EncType> getEncTypeSet(EncType type) {
+        switch(type) {
+          case ECIES_X25519:
+              return LeaseSetKeys.SET_EC;
+          case MLKEM512_X25519:
+              return LeaseSetKeys.SET_PQ1;
+          case MLKEM768_X25519:
+              return LeaseSetKeys.SET_PQ2;
+          case MLKEM1024_X25519:
+              return LeaseSetKeys.SET_PQ3;
+          default:
+              throw new IllegalArgumentException("No pattern for " + type);
+        }
+    }
+
     /**
      * scenario 1: New Session Message
      *
@@ -383,8 +531,12 @@ public final class ECIESAEADEngine {
         System.arraycopy(pk.getData(), 0, data, 0, KEYLEN);
 
         HandshakeState state;
+        EncType type = targetPrivateKey.getType();
         try {
-            state = new HandshakeState(HandshakeState.PATTERN_ID_IK, HandshakeState.RESPONDER, _edhThread);
+            String pattern = getNoisePattern(type);
+            // Bob does not need a key factory
+            //state = new HandshakeState(pattern, HandshakeState.RESPONDER, _edhThread, getHybridKeyFactory(type));
+            state = new HandshakeState(pattern, HandshakeState.RESPONDER, _edhThread);
         } catch (GeneralSecurityException gse) {
             throw new IllegalStateException("bad proto", gse);
         }
@@ -392,9 +544,13 @@ public final class ECIESAEADEngine {
                                         targetPrivateKey.toPublic().getData(), 0);
         state.start();
         if (_log.shouldDebug())
-            _log.debug("State before decrypt new session: " + state);
+            _log.debug("State before decrypt new session (" + data.length + " bytes) " + state);
 
         int payloadlen = data.length - (KEYLEN + KEYLEN + MACLEN + MACLEN);
+        DHState hyb = state.getRemoteHybridKeyPair();
+        if (hyb != null) {
+            payloadlen -= hyb.getPublicKeyLength() + MACLEN;
+        }
         byte[] payload = new byte[payloadlen];
         try {
             state.readMessage(data, 0, data.length, payload, 0);
@@ -402,7 +558,7 @@ public final class ECIESAEADEngine {
             // we'll get this a lot on muxed SKM
             // logged at INFO in caller
             if (_log.shouldDebug())
-                _log.debug("Decrypt fail NS, state at failure: " + state, gse);
+                _log.debug("Decrypt fail NS " + data.length + " bytes, state at failure: " + state, gse);
             // restore original data for subsequent ElG attempt
             System.arraycopy(xx, 0, data, 0, KEYLEN - 1);
             data[KEYLEN - 1] = xx31;
@@ -467,7 +623,7 @@ public final class ECIESAEADEngine {
             state.destroy();
         } else {
             // tell the SKM
-            PublicKey alice = new PublicKey(EncType.ECIES_X25519, alicePK);
+            PublicKey alice = new PublicKey(type, alicePK);
             keyManager.createSession(alice, null, state, null);
             setResponseTimerNS(alice, pc.cloveSet, keyManager);
         }
@@ -638,8 +794,13 @@ public final class ECIESAEADEngine {
         state.mixHash(tag, 0, TAGLEN);
         if (_log.shouldDebug())
             _log.debug("State after mixhash tag before decrypt new session reply: " + state);
+        int tmplen = 48;
+        DHState hyb = state.getRemoteHybridKeyPair();
+        if (hyb != null) {
+            tmplen += hyb.getPublicKeyLength() + MACLEN;
+        }
         try {
-            state.readMessage(data, 8, 48, ZEROLEN, 0);
+            state.readMessage(data, 8, tmplen, ZEROLEN, 0);
         } catch (GeneralSecurityException gse) {
             if (_log.shouldWarn()) {
                 _log.warn("Decrypt fail NSR part 1", gse);
@@ -667,9 +828,16 @@ public final class ECIESAEADEngine {
         byte[] encpayloadkey = new byte[32];
         _hkdf.calculate(split.k_ba.getData(), ZEROLEN, INFO_6, encpayloadkey);
         rcvr.initializeKey(encpayloadkey, 0);
-        byte[] payload = new byte[data.length - (TAGLEN + KEYLEN + MACLEN + MACLEN)];
+        int off = TAGLEN + KEYLEN + MACLEN;
+        int plen = data.length - (TAGLEN + KEYLEN + MACLEN + MACLEN);
+        if (hyb != null) {
+            int len = hyb.getPublicKeyLength() + MACLEN;
+            off += len;
+            plen -= len;
+        }
+        byte[] payload = new byte[plen];
         try {
-            rcvr.decryptWithAd(hash, data, TAGLEN + KEYLEN + MACLEN, payload, 0, payload.length + MACLEN);
+            rcvr.decryptWithAd(hash, data, off, payload, 0, plen + MACLEN);
         } catch (GeneralSecurityException gse) {
             if (_log.shouldWarn()) {
                 _log.warn("Decrypt fail NSR part 2", gse);
@@ -714,7 +882,7 @@ public final class ECIESAEADEngine {
         }
 
         // tell the SKM
-        PublicKey bob = new PublicKey(EncType.ECIES_X25519, bobPK);
+        PublicKey bob = new PublicKey(keyManager.getType(), bobPK);
         keyManager.updateSession(bob, oldState, state, null, split);
 
         if (pc == null)
@@ -872,8 +1040,7 @@ public final class ECIESAEADEngine {
     private byte[] x_encrypt(CloveSet cloves, PublicKey target, Destination to, PrivateKey priv,
                              RatchetSKM keyManager,
                              ReplyCallback callback) {
-        if (target.getType() != EncType.ECIES_X25519)
-            throw new IllegalArgumentException();
+        checkType(target.getType());
         if (Arrays.equals(target.getData(), NULLPK)) {
             if (_log.shouldWarn())
                 _log.warn("Zero static key target");
@@ -904,6 +1071,7 @@ public final class ECIESAEADEngine {
             }
             if (_log.shouldDebug())
                 _log.debug("Encrypting as NSR to " + target + " with tag " + re.tag.toBase64());
+// trash old state if this throws IAE???
             return encryptNewSessionReply(cloves, target, state, re.tag, keyManager, callback);
         }
         byte rv[] = encryptExistingSession(cloves, target, re, callback, keyManager);
@@ -943,9 +1111,13 @@ public final class ECIESAEADEngine {
     private byte[] encryptNewSession(CloveSet cloves, PublicKey target, Destination to, PrivateKey priv,
                                      RatchetSKM keyManager,
                                      ReplyCallback callback) {
+        EncType type = target.getType();
+        if (type != priv.getType())
+            throw new IllegalArgumentException("Key mismatch " + target + ' ' + priv);
         HandshakeState state;
         try {
-            state = new HandshakeState(HandshakeState.PATTERN_ID_IK, HandshakeState.INITIATOR, _edhThread);
+            String pattern = getNoisePattern(target.getType());
+            state = new HandshakeState(pattern, HandshakeState.INITIATOR, _edhThread, getHybridKeyFactory(type));
         } catch (GeneralSecurityException gse) {
             throw new IllegalStateException("bad proto", gse);
         }
@@ -962,7 +1134,12 @@ public final class ECIESAEADEngine {
 
         byte[] payload = createPayload(cloves, cloves.getExpiration(), NS_OVERHEAD);
 
-        byte[] enc = new byte[KEYLEN + KEYLEN + MACLEN + payload.length + MACLEN];
+        int enclen = KEYLEN + KEYLEN + MACLEN + payload.length + MACLEN;
+        DHState hyb = state.getLocalHybridKeyPair();
+        if (hyb != null) {
+            enclen += hyb.getPublicKeyLength() + MACLEN;
+        }
+        byte[] enc = new byte[enclen];
         try {
             state.writeMessage(enc, 0, payload, 0, payload.length);
         } catch (GeneralSecurityException gse) {
@@ -1071,7 +1248,12 @@ public final class ECIESAEADEngine {
         byte[] payload = createPayload(cloves, 0, NSR_OVERHEAD);
 
         // part 1 - tag and empty payload
-        byte[] enc = new byte[TAGLEN + KEYLEN + MACLEN + payload.length + MACLEN];
+        int enclen = TAGLEN + KEYLEN + MACLEN + payload.length + MACLEN;
+        DHState hyb = state.getLocalHybridKeyPair();
+        if (hyb != null) {
+            enclen += hyb.getPublicKeyLength() + MACLEN;
+        }
+        byte[] enc = new byte[enclen];
         System.arraycopy(tag, 0, enc, 0, TAGLEN);
         try {
             state.writeMessage(enc, TAGLEN, ZEROLEN, 0, 0);
@@ -1103,8 +1285,12 @@ public final class ECIESAEADEngine {
         byte[] encpayloadkey = new byte[32];
         _hkdf.calculate(split.k_ba.getData(), ZEROLEN, INFO_6, encpayloadkey);
         sender.initializeKey(encpayloadkey, 0);
+        int off = TAGLEN + KEYLEN + MACLEN;
+        if (hyb != null) {
+            off += hyb.getPublicKeyLength() + MACLEN;
+        }
         try {
-            sender.encryptWithAd(hash, payload, 0, enc, TAGLEN + KEYLEN + MACLEN, payload.length);
+            sender.encryptWithAd(hash, payload, 0, enc, off, payload.length);
         } catch (GeneralSecurityException gse) {
             if (_log.shouldWarn())
                 _log.warn("Encrypt fail NSR part 2", gse);
@@ -1488,7 +1674,7 @@ public final class ECIESAEADEngine {
                 return;
             if (!ls2.isCurrent(Router.CLOCK_FUDGE_FACTOR))
                 continue;
-            PublicKey pk = ls2.getEncryptionKey(LeaseSetKeys.SET_EC);
+            PublicKey pk = ls2.getEncryptionKey(getEncTypeSet(from.getType()));
             if (!from.equals(pk))
                 continue;
             if (!ls2.verifySignature())
