@@ -27,7 +27,6 @@ import net.i2p.data.router.RouterInfo;
 import net.i2p.router.CommSystemFacade;
 import net.i2p.router.OutNetMessage;
 import net.i2p.router.RouterContext;
-import net.i2p.router.transport.crypto.DHSessionKeyBuilder;
 import net.i2p.router.transport.crypto.X25519KeyFactory;
 import net.i2p.router.transport.udp.UDPTransport;
 import net.i2p.router.util.EventLog;
@@ -452,16 +451,6 @@ public class CommSystemFacadeImpl extends CommSystemFacade {
         _manager.stopAndUnregister(t);
     }
 
-    /**
-     *  Hook for pluggable transport creation.
-     *
-     *  @since 0.9.16
-     */
-    @Override
-    public DHSessionKeyBuilder.Factory getDHFactory() {
-        return _manager.getDHFactory();
-    }
-
     /**
      *  Factory for making X25519 key pairs.
      *  @since 0.9.46
